#!/usr/bin/env python
from CeboMsrApiPython import LibraryInterface, DeviceType, Trigger
import time

# CeboMsr - Python example - Triggers.
#
# This example outlines how two CeboMsr devices can be chained via trigger.
# The first device is used as master, it is configured to output an alternating trigger,
# while the second device is configured to respond to this trigger.
# Both devices sample digital port #0 and single ended input #0.
# Trigger #0 is used for both master and slave.
#
# Requirements: Two CeboLC devices must be connected to the host, GND and TRG must be
# connected between both devices.
#
# This example works with CeboLC only, but can be ported to Cebo-Stick as well.
#
# @author Thomas Hoppe

def dumpFrames(device, frames):
    """
    Write frames to console.
    device: Device string.
    frames: Frame to dump.
    """
    for f in frames:
        print("%s: se#0: %.2f V, dp#0: %d" % (
            device, f.getSingleEnded(0), f.getDigitalPort(0)))

def runDataAcquisition(master, slave):
    """
    Start DAQ, read frames and output the results.
    master: Master device.
    slave: Slave device.
    """
    # The slave must be started first, as it reacts on the master's
    # trigger. Continuous DAQ is used. Timed using an external trigger.
    slave.startContinuousExternalTimedDataAcquisition()

    # The master uses hardware timed DAQ, continuous at 50 Hz.
    # The 'false' here signals that the process should start immediately.
    master.startContinuousDataAcquisition(50, False)

    # The example reads at least 10 frames from
    # both devices and subsequently output the samples.
    masterFrames, slaveFrames = 0, 0
    while (masterFrames < 10 or slaveFrames < 10):
        # Start by reading frames from master,
        # output it and increment counter.
        frames = master.readNonBlocking()
        dumpFrames("master", frames)
        masterFrames += len(frames)

        # Do the same with the slave.
        frames = slave.readNonBlocking()
        dumpFrames("slave", frames)
        slaveFrames += len(frames)

        # Don't poll to frequent, this would fully utilize one core.
        time.sleep(0.001)

    # Finished, gracefully stop DAQ.
    slave.stopDataAcquisition()
    master.stopDataAcquisition()

def configure(master, slave):
    """
    Both devices are fully configured here.
    master: Master device.
    slave: Slave device.
    """
    # The trigger for the master must be set to alternating output.
    master.getTriggers()[0].setConfig(
        Trigger.TriggerConfig.OutputAlternating)

    # The slave's trigger must be set to alternating as well.
    slave.getTriggers()[0].setConfig(
        Trigger.TriggerConfig.InputAlternating)

    # Both devices now gets configured to the same input frame layout.
    master.setupInputFrame([
        master.getSingleEndedInputs()[0],
        master.getDigitalPorts()[0]
    ])

    # ... slave
    slave.setupInputFrame([
        slave.getSingleEndedInputs()[0],
        slave.getDigitalPorts()[0]
    ])

def main():
    """
    The examples main method.
    """
    try:
        # Search for devices ...
        devices = LibraryInterface.enumerate(DeviceType.CeboLC)
        if (len(devices) != 2):
            print("Exactly two devices are required.")
            return

        # As both devices can act as master,
        # we simply use the first one for this role.
        master = devices[0]
        master.open()
        print("Master: " + master.getSerialNumber() +
                "@" + master.getIdentifier())

        # ... and the second as slave.
        slave = devices[1]
        slave.open()
        print("Slave: " + slave.getSerialNumber() +
                "@" + slave.getIdentifier())

        # Configure both master and slave ...
        configure(master, slave)

        # ... and do the DAQ.
        runDataAcquisition(master, slave)

        # Close both.
        master.close()
        slave.close()
    except Exception as e:
        print(e)

if (__name__ == "__main__"):
    main()
