package com.cesys.cebo.cebomsr;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * Offers functionality to work with digital ports. Direct construction
 * is not possible. Instances can be retrieved calling getDigitalPorts() of
 * the respective {@link Device} instance.
 *
 * @author Thomas Hoppe
 */
public final class DigitalPort implements Input, Output {
	private final Device _device;
	private final int _interfaceId;
	private final int _index;
	private final int _ioCount;
	private final String _name;
	private final List<String> _ioNames = new ArrayList<String>();

	int getInterfaceId() {
		return _interfaceId;
	}

	int getIndex() {
		return _index;
	}

	DigitalPort(Device device, int interfaceId, int index) throws IOException {
		_device = device;
		_interfaceId = interfaceId;
		_index = index;
		int[] r = new int[1];
		SystemConnector.checkError(SystemConnector.CeboMsrGetDigitalPortIoCount(device.getHandle(), getInterfaceId(), r));
		_ioCount = r[0];
		byte[] tmp = new byte[SystemConnector.MaxIdentLength];
		SystemConnector.checkError(SystemConnector.CeboMsrGetPeripheralName(device.getHandle(), interfaceId, tmp));
		_name = Utils.convertByteArrayToString(tmp);
		for (int i = 0; i < _ioCount; ++i) {
			SystemConnector.checkError(SystemConnector.CeboMsrGetDigitalPortIoName(device.getHandle(), interfaceId, i, tmp));
			_ioNames.add(Utils.convertByteArrayToString(tmp));
		}
	}

	/**
	 * Set output mask. The default mask is 0.
	 * @param mask Mask to use, bits that have state '1' specify output direction.
	 */
	public void setOutputEnableMask(int mask) throws IOException {
		SystemConnector.checkError(SystemConnector.CeboMsrSetDigitalPortOutputEnableMask(_device.getHandle(), getInterfaceId(), mask));
	}

	/**
	 * Return count of I/O's for the specified port.
	 */
	public int getIoCount() {
		return _ioCount;
	}

	/**
	 * Read value currently active on the port.
	 */
	public int read() throws IOException {
		int r[] = new int[1];
		SystemConnector.checkError(SystemConnector.CeboMsrReadDigitalPort(_device.getHandle(), getInterfaceId(), r));
		return r[0];
	}

	/**
	 * Set output I/O's. Only the bits that have been defined as output
	 * using setOutputEnableMask() are modified.
	 * @param value Value to set on this port.
	 */
	public void write(int value) throws IOException {
		SystemConnector.checkError(SystemConnector.CeboMsrWriteDigitalPort(_device.getHandle(), getInterfaceId(), value));
	}

	/**
	 * Returns name of the component.
	 */
	public String getName() {
		return _name;
	}

	/**
	 * Return name of the I/O as specified by io.
	 * @param io I/O index in range (0 <= io < getIoCount()).
	 */
	public String getIoName(int io) {
		return _ioNames.get(io);
	}

	public String toString() {
		return "DigitalPort" + _index;
	}
}
